#include <stdlib.h>
#include <stdio.h>
#include <stdbool.h>
#include <string.h>
#include "hufftree.h"



/** \brief Fonction de parcours rcursif de l'arbre de Huffman permettant
    de remplir la table des codes. Elle prend en argument un buffer dans
    lequel elle va stocker les diffrents "bits" de codage rencontrs au cours
    de la descente et recopiera, une fois arrive  un feuille, la chane de
    ces "bits" dans l'entre de la table correspondant au caractre de la
    feuille.
    \param tree : Noeud courant explor de l'arbre.
    \param accu : Buffer pour stocker au fur et  mesure de la descente dans
    l'arbre les caractres '0' et '1' des bits rencontrs.
    \param accu_len : Nombre actuel de caractres dans l'accu.
    \param codes : Table des codes  remplir.
    \return Boolen disant si la mmorisation des code s'est passe
    correctement.
    \note Cette fonction est locale au fichier courant et n'est pas exporte.
    Elle sert uniquement de "helper"  la fonction \a get_codes. */
static bool tree_walk (struct huff_node_t *tree, char *accu, int accu_len,
                       char **codes)
{
  if (tree != NULL) {
    if (tree->left == NULL) {
      /* Noeud terminal. Notons que par invariant le fils droit doit tre
         galement  NULL. */
      codes[tree->letter] = malloc ((accu_len + 1) * sizeof (char)) ;
      if (codes[tree->letter] == NULL) return (false) ;
      strncpy (codes[tree->letter], accu, accu_len) ;
      codes[tree->letter][accu_len] = '\0' ;
      return (true) ;
    }
    else {
      /* Noeud non terminal, on rcurse  gauche et  droite en augmentant
         l'accumulateur. */
      accu[accu_len] = '0' ;
      if (! tree_walk (tree->left, accu, accu_len + 1, codes)) return (false) ;
      accu[accu_len] = '1' ;
      return (tree_walk (tree->right, accu, accu_len + 1, codes)) ;
    }
  }
  return (true) ;
}



/** \brief Rcupration de la table des codes associs  l'arbre de Huffman
    pass en argument.
    \param tree : Pointeur sur la racine de l'arbre de Huffman.
    \return Tableau de 256 chanes de caractres reprsentant le code associ
     chaque caractre (indexation par le code du caractre). Chaque chane
    est compose de caractres '0' et '1' formant une reprsentation textuelle
    du code binaire associ au caractre encod. */
char** get_codes (struct huff_node_t *tree)
{
  int i ;
  /* Le cas le plus dgnr le code le plus long est gal au nombre de
     symboles (ici 256) - 1. Il faut 1 octet de plus pour clore la chane
     obtenue. Donc une longueur de chane maximale de 256 caractres. */
  char tmp_buffer[256] ;

  char **codes = malloc (257 * sizeof (char*)) ;
  if (codes == NULL) return (NULL) ;

  /* Initialisation des codes  "aucune chane". */
  for (i = 0; i < 256; i++) codes[i] = NULL ;

  /* Parcours rcursif de l'arbre pour remplir la table. */
  if (! tree_walk (tree, tmp_buffer, 0, codes))
    printf ("Error while building codes table. Table may be incomplete.\n") ;

  return (codes) ;
}
